'use strict';

const sequelize = require('sequelize');
const _ = require('lodash');
const index = require('./index.cjs');

const selectMap = (index, type, func) => {
  let result = [];
  if (type === "page") {
    const _index = index;
    if (typeof func === "function" && _index.page.total > 0 && _index.contents && _index.contents.length > 0) {
      result = _index.contents.map(func);
    }
  } else if (type === "all") {
    const _index = index;
    if (typeof func === "function" && _.size(_index) > 0) {
      _.map(_index, (items) => {
        result.push(func(items));
      });
    }
  } else if (type === "row") {
    const _index = index;
    if (typeof func === "function" && _index) {
      result = [func(_index)];
    }
  }
  if (result.length > 0 && result[0] && typeof result[0].then === "function") {
    result = Promise.all(result);
  }
  return result;
};
class SequelizeDB {
  constructor(sequelize) {
    this.fn = null;
    this.sequelize = sequelize;
    this.dbqb = new index(this);
  }
  async getTables() {
    return await this.sequelize.query("SHOW TABLES", {
      type: sequelize.QueryTypes.SHOWTABLES
    });
  }
  async getFields(table) {
    return await this.sequelize.query(`SHOW FIELDS FROM ${table}`, {
      type: sequelize.QueryTypes.SELECT,
      raw: true
    });
  }
  validate(fn) {
    this.fn = fn;
  }
  async selectQuery(active) {
    return this.dbqb.selectQuery(active);
  }
  async countQuery(active) {
    return this.dbqb.countQuery(active);
  }
  queryOptions(t = null) {
    let options = {
      raw: true
    };
    if (t instanceof sequelize.Transaction) {
      options.transaction = t;
    } else if (t) {
      options = {
        ...options,
        ...t
      };
    }
    return options;
  }
  async transaction(t = null) {
    if (t instanceof sequelize.Transaction) {
      return [t, t, false];
    } else if (t) {
      const opt = {
        ...t
      };
      let gen = false;
      if (!opt.transaction) {
        opt.transaction = await this.sequelize.transaction();
        gen = true;
      }
      return [opt.transaction, opt, gen];
    } else {
      const trans = await this.sequelize.transaction();
      return [trans, trans, true];
    }
  }
  setQueryOptions(t, qo) {
    if (t instanceof sequelize.Transaction) {
      return {
        ...qo,
        transaction: t
      };
    } else if (t) {
      return {
        ...t,
        ...qo
      };
    }
    return null;
  }
  async queryRow(query, t = null) {
    const selectOption = this.queryOptions(t);
    selectOption.type = sequelize.QueryTypes.SELECT;
    const index = await this.sequelize.query(query, selectOption);
    if (_.size(index) === 0) {
      return null;
    }
    const row = index[0];
    if (this.fn) {
      _.forEach(row, (val, key) => {
        if (this.fn) {
          row[key] = this.fn(val, key);
        }
      });
    }
    return row;
  }
  async queryAll(query, t = null) {
    const selectOption = this.queryOptions(t);
    selectOption.type = sequelize.QueryTypes.SELECT;
    const index = await this.sequelize.query(query, selectOption);
    if (_.size(index) === 0) {
      return null;
    }
    if (this.fn) {
      _.forEach(index, (items, idx) => {
        _.forEach(items, (item, key) => {
          if (this.fn) {
            index[idx][key] = this.fn(item, key);
          }
        });
      });
    }
    return index;
  }
  async queryOne(query, t = null) {
    const selectOption = this.queryOptions(t);
    selectOption.type = sequelize.QueryTypes.SELECT;
    const index = await this.sequelize.query(query, selectOption);
    if (_.size(index) === 0) {
      return null;
    }
    const keys = _.keys(index[0]);
    return index[0][keys[0]];
  }
  async selectRow(active, t = null) {
    active.limit = 1;
    const sQuery = await this.dbqb.selectQuery(active);
    if (!sQuery) {
      return null;
    }
    return this.queryRow(sQuery, t);
  }
  async selectAll(active, t = null) {
    if (!active.nolimit) {
      active.offset = active.offset || 0;
      active.limit = active.limit || 1;
    }
    const sQuery = await this.dbqb.selectQuery(active);
    if (!sQuery) {
      return null;
    }
    return this.queryAll(sQuery, t);
  }
  async selectOne(active, t = null) {
    active.limit = 1;
    const sQuery = await this.dbqb.selectQuery(active);
    if (!sQuery) {
      return null;
    }
    return this.queryOne(sQuery, t);
  }
  async selectPage(active, t = null) {
    if (!active.nolimit) {
      active.offset = active.offset || 0;
      active.limit = active.limit || 20;
    }
    let sQuery = "";
    let sQueryCnt = "";
    if (active.query && active.queryCnt) {
      sQuery = active.query;
      sQueryCnt = active.queryCnt;
      if (!active.nolimit) {
        sQuery += ` LIMIT ${active.offset}, ${active.limit}`;
      }
    } else {
      sQuery = await this.dbqb.selectQuery(active);
      if (active.having && _.size(active.having) > 0 || active.havingOr && _.size(active.havingOr) > 0) {
        const activeCnt = { ...active };
        _.unset(activeCnt, "limit");
        _.unset(activeCnt, "offset");
        const sQuery2 = await this.dbqb.selectQuery(activeCnt);
        sQueryCnt = `SELECT COUNT(1) FROM ( ${sQuery2} ) AS cnt`;
      } else {
        sQueryCnt = await this.dbqb.countQuery(active);
      }
    }
    const aReturn = {};
    if (sQuery && sQueryCnt) {
      const aContents = await this.queryAll(sQuery, t);
      let nCount = _.toNumber(await this.queryOne(sQueryCnt, t));
      if (_.isNaN(nCount)) {
        nCount = 0;
      }
      const page = {
        offset: 0,
        limit: 0,
        total: 0,
        page: 1,
        lastPage: 1
      };
      if (active.nolimit) {
        page.offset = 0;
        page.limit = nCount;
        page.total = nCount;
        page.page = 1;
        page.lastPage = 1;
      } else {
        page.offset = _.toNumber(active.offset);
        page.limit = _.toNumber(active.limit);
        page.total = nCount;
        page.page = Math.floor(page.offset / page.limit) + 1;
        page.lastPage = Math.ceil(page.total / page.limit);
      }
      const nContentCnt = _.size(aContents);
      if (nContentCnt > 0 && nContentCnt > page.total) {
        page.total = nContentCnt;
      }
      aReturn.page = page;
      aReturn.contents = aContents;
    } else {
      aReturn.page = {
        offset: 0,
        limit: 0,
        total: 0,
        page: 1,
        lastPage: 1
      };
      aReturn.contents = null;
    }
    return aReturn;
  }
  async select(_active, type = "page", func, t) {
    const active = { ..._active };
    if (active.func) {
      func = active.func;
    }
    let promise = [];
    let index = null;
    if (type === "page") {
      const pageIndex = await this.selectPage(active, t);
      if (pageIndex && _.isArray(pageIndex.contents) && _.size(pageIndex.contents) > 0 && typeof func === "function") {
        promise = _.map(pageIndex.contents, (item) => {
          if (func) {
            func(item, active);
          }
        });
      }
      index = pageIndex;
    } else if (type === "all") {
      const allIndex = await this.selectAll(active, t);
      if (allIndex && _.size(allIndex) > 0 && typeof func === "function") {
        _.map(allIndex, (items) => {
          if (func) {
            promise.push(func(items, active));
          }
        });
      }
      index = allIndex;
    } else if (type === "row") {
      const rowIndex = await this.selectRow(active, t);
      if (rowIndex && typeof func === "function") {
        promise.push(func(rowIndex, active));
      }
      index = rowIndex;
    }
    if (_.size(promise) > 0 && promise[0] && typeof promise[0].then === "function") {
      await Promise.all(promise);
    }
    return index;
  }
  async insert(active, t = null) {
    const query = await this.dbqb.insertQuery(active);
    if (query === null || !query) {
      throw new Error("query builder error");
    }
    const insertOption = this.queryOptions(t);
    insertOption.type = sequelize.QueryTypes.INSERT;
    const index = await this.sequelize.query(query, insertOption);
    return {
      result: true,
      insertId: index[0]
    };
  }
  async insertAll(active, t = null) {
    const query = await this.dbqb.insertAllQuery(active);
    if (query === null || !query) {
      throw new Error("query builder error");
    }
    if (!active.table) {
      return;
    }
    const insertOption = this.queryOptions(t);
    insertOption.type = sequelize.QueryTypes.INSERT;
    const index = await this.sequelize.query(query, insertOption);
    let priKey = null;
    const _tField = await this.getFields(active.table);
    for (const _item of _tField) {
      if (_item.Key === "PRI" && _item.Extra === "auto_increment") {
        priKey = _item.Field;
        break;
      }
    }
    let insertIds = [];
    if (priKey) {
      const insertId = index[0];
      const nDataCnt = _.size(active.data);
      const _idxs = await this.selectAll({
        table: active.table,
        field: [priKey],
        where: {
          [`${priKey} >=`]: insertId
        },
        limit: nDataCnt,
        orderBy: {
          [`${priKey} >=`]: "ASC"
        }
      }, t);
      insertIds = _.map(_idxs, (id) => id[priKey]);
    }
    return {
      result: true,
      insertId: insertIds
    };
  }
  async update(active, t = null) {
    const query = await this.dbqb.updateQuery(active);
    if (query === null || !query) {
      throw new Error("query builder error");
    }
    const updateOption = this.queryOptions(t);
    updateOption.type = sequelize.QueryTypes.UPDATE;
    const index = await this.sequelize.query(query, updateOption);
    return {
      result: true,
      affected: index[1]
    };
  }
  async insertUpdate(active, t = null) {
    const query = await this.dbqb.insertUpdateQuery(active);
    if (query === null || !query) {
      throw new Error("query builder error");
    }
    const insertOption = this.queryOptions(t);
    insertOption.type = sequelize.QueryTypes.INSERT;
    await this.sequelize.query(query, insertOption);
    return {
      result: true
    };
  }
  async delete(active, t = null) {
    const query = await this.dbqb.deleteQuery(active);
    if (query === null || !query) {
      throw new Error("query builder error");
    }
    const deleteOption = this.queryOptions(t);
    deleteOption.type = sequelize.QueryTypes.DELETE;
    await this.sequelize.query(query, deleteOption);
    return {
      result: true
    };
  }
  getErrorLogs() {
    return this.dbqb.getErrorLogs();
  }
}

exports.SequelizeDB = SequelizeDB;
exports.selectMap = selectMap;
