import { Transaction, QueryOptions, Sequelize } from 'sequelize';
import { IActive, IFieldItem } from './index.js';

interface ISelectPage {
    offset: number;
    limit: number;
    total: number;
    page: number;
    lastPage: number;
}
interface ISelectPageResult<T = any> {
    contents: T[] | null;
    page: ISelectPage;
}
type TSelectResult<T = any> = T | T[] | ISelectPageResult<T>;
type TSelectFn<T> = (item: T, active: IActive) => Promise<void> | void;
type TQueryOptions = Transaction | QueryOptions | null;
type TSelectTypes = 'row' | 'all' | 'page';
interface ISelectActive<T = any> extends IActive {
    func?: TSelectFn<T>;
    nolimit?: boolean;
    query?: string;
    queryCnt?: string;
}
declare const selectMap: <T = any, TResult = any>(index: TSelectResult<T>, type: TSelectTypes, func: (item: T) => TResult) => TResult[];
declare class SequelizeDB {
    readonly sequelize: Sequelize;
    private dbqb;
    private fn;
    constructor(sequelize: Sequelize);
    getTables(): Promise<string[]>;
    getFields(table: string): Promise<IFieldItem[]>;
    validate(fn: (item: any, key: string) => void): void;
    selectQuery(active: IActive): Promise<string | null>;
    countQuery(active: IActive): Promise<string | null>;
    private queryOptions;
    transaction(t?: TQueryOptions): Promise<[Transaction, TQueryOptions, boolean]>;
    setQueryOptions(t: TQueryOptions, qo: QueryOptions): TQueryOptions;
    queryRow<T = any>(query: string, t?: TQueryOptions): Promise<T | null>;
    queryAll<T = any>(query: string, t?: TQueryOptions): Promise<T[] | null>;
    queryOne<T = any>(query: string, t?: TQueryOptions): Promise<T | null>;
    selectRow<T = any>(active: ISelectActive, t?: TQueryOptions): Promise<T | null>;
    selectAll<T = any>(active: ISelectActive, t?: TQueryOptions): Promise<T[] | null>;
    selectOne<T = any>(active: ISelectActive, t?: TQueryOptions): Promise<T | null>;
    selectPage<T = any>(active: ISelectActive, t?: TQueryOptions): Promise<ISelectPageResult<T>>;
    select<T = any>(_active: ISelectActive<T>, type?: 'page', func?: TSelectFn<T> | null, t?: TQueryOptions): Promise<ISelectPageResult<T> | null>;
    select<T = any>(_active: ISelectActive<T>, type: 'all', func?: TSelectFn<T> | null, t?: TQueryOptions): Promise<T[] | null>;
    select<T = any>(_active: ISelectActive<T>, type: 'row', func?: TSelectFn<T> | null, t?: TQueryOptions): Promise<T | null>;
    select<T = any>(_active: ISelectActive<T>, type: TSelectTypes, func?: TSelectFn<T> | null, t?: TQueryOptions): Promise<TSelectResult<T> | null>;
    insert(active: IActive, t?: TQueryOptions): Promise<{
        result: boolean;
        insertId: number;
    }>;
    insertAll(active: IActive, t?: TQueryOptions): Promise<{
        result: boolean;
        insertId: number[];
    } | undefined>;
    update(active: IActive, t?: TQueryOptions): Promise<{
        result: boolean;
        affected: number;
    }>;
    insertUpdate(active: IActive, t?: TQueryOptions): Promise<{
        result: boolean;
    }>;
    delete(active: IActive, t?: Transaction | null): Promise<{
        result: boolean;
    }>;
    getErrorLogs(): string[];
}

export { ISelectActive, ISelectPage, ISelectPageResult, SequelizeDB, TQueryOptions, TSelectFn, TSelectResult, TSelectTypes, selectMap };
